/* automake.h - -*- C++ -*- classes to work with Makefile.am
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifndef _AUTOMAKE_H
#define _AUTOMAKE_H

#include <string>
#include <list>

#include "iexceptions.h"

class ETargetNotFound : public Exception
{
public:
	ETargetNotFound(const string &target) : Exception("Target not found: " + target)
	{ }; 
};

class Target;

typedef list<string> target_values;
typedef list<Target> target_list;

typedef list<string> script_list;

class Target
{
private:
	target_values values;
	string name;
public:
	typedef target_values::const_iterator value_iterator;
	
	Target(const string &str);
	Target(const Target &target);

	string get_name(void) const;
	
	void add_value(const string &value);
	void remove_value(const string &value);
	bool has_value(const string &value) const;

	int values_count(void) const;
	
	value_iterator get_value_iterator(void) const;
	bool iterator_last(value_iterator iter) const;

	friend bool operator!=(const Target &target,const string &str)
	{
		return target.name != str;
	};
};

class Makefile
{
private:
	string dir;

	target_list targets;
	script_list scripts;

	void load(void);
public:
	typedef target_list::iterator target_iterator;
	Makefile(const string &_dir);

	void save(void) throw(EIOError);

	const Target &find_target(const string &target);
	void add_target(const string &target,const string &values);
	void remove_target(const string &target);

	void add_to_target(const string &target,const string &value);
	void remove_from_target(const string &target,const string &value);
	
	target_iterator get_target_iterator(void);
	bool iterator_last(target_iterator iter);
};

#endif
