/* conffile.cc - -*- C++ -*- class that utilizes gnome-config.h functions
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include "conffile.h"

ConfFile::ConfFile(const string &file)
{
	prefix = "=" + file + "=";
}

void ConfFile::read(const string &path,int &value)
{
	string ipath = prefix + "/" + path;
	value = gnome_config_get_int(ipath.c_str());
}
	 
void ConfFile::read(const string &path,string &value)
{
	string ipath = prefix + "/" + path;
	char *ret = gnome_config_get_string(ipath.c_str());
	value = ret?ret:"";
}

void ConfFile::read(const string &path,bool &value)
{
	string ipath = prefix + "/" + path;
	value = gnome_config_get_bool(ipath.c_str());
}

void ConfFile::write(const string &path,const int &value)
{
	string ipath = prefix + "/" + path;
	gnome_config_set_int(ipath.c_str(),value);
}

void ConfFile::write(const string &path,const string &value)
{
	string ipath = prefix + "/" + path;
	gnome_config_set_string(ipath.c_str(),value.c_str());
}

void ConfFile::write(const string &path,const bool &value)
{
	string ipath = prefix + "/" + path;
	gnome_config_set_bool(ipath.c_str(),value);
}

void ConfFile::drop(void)
{
	char str[384];

	sprintf(str,"%s",prefix.c_str());
	
	gnome_config_sync_file(str);
	gnome_config_drop_file(prefix.c_str());
}

void ConfFile::clear_section(const string &sect)
{
	string path = prefix + "/" + sect;
	gnome_config_clean_section(path.c_str());
}

ConfFile::iterator ConfFile::get_iterator(const string &sect)
{
	string path = prefix + "/" + sect;
	return gnome_config_init_iterator(path.c_str());
}

ConfFile::iterator ConfFile::iterator_next(iterator iter,string &key,string &value)
{
	char *ikey, *ivalue;
	iterator it = gnome_config_iterator_next(iter,&ikey,&ivalue);
	key = ikey?ikey:"";
	value = ivalue?ivalue:"";

	return it;
}
