/* dialogs.cc - Common dialog classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "dialogs.h"

//GladeWidgetOwner
GtkWidget *GladeWidgetOwner::get_widget(const string &name) const
{
	return glade_xml_get_widget(xml,name.c_str());
}

//Dialog
Dialog::Dialog(const string &file,const string &name)
{
	string real_name = string(PACKAGE_DATA_DIR) + "/" + PACKAGE + "/glade/" + file + ".glade";

	xml = glade_xml_new(real_name.c_str(),name.c_str());
	
	//if(!xml)
	//	  throw EIOError("Interface file loading failed.",real_name);
	
	window = get_widget(name);	
}

Dialog::~Dialog()
{
	if(GTK_IS_WIDGET(window))
		gtk_widget_destroy(window);
}

int Dialog::run(void)
{
	gtk_widget_show(window);

	return 0;
}

//GtkedDialog
int GtkedDialog::run(void)
{
	gtk_widget_show(window);

	gtk_main();
	
	return GTK_IS_WIDGET(window);
}

//GnomedDialog
int GnomedDialog::run(void)
{
	int i = gnome_dialog_run(GNOME_DIALOG(window));
	gtk_widget_hide(window);
	return i;
}

//DruidDialog
void DruidDialog::on_druid_cancel(GnomeDruid *druid,DruidDialog *dia)
{
	gtk_widget_destroy(dia->window);
	gtk_main_quit();
}

void DruidDialog::on_pagefinish_finish(GnomeDruidPage *druid_page,GnomeDruid *druid,DruidDialog *dia)
{
	gtk_widget_hide(dia->window);
	gtk_main_quit();
}

void DruidDialog::connect_signals(void)
{
	GtkedDialog::connect_signals();
	
	gtk_signal_connect(GTK_OBJECT(get_widget("druid")),"cancel",GTK_SIGNAL_FUNC(on_druid_cancel),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("pagefinish")),"finish",GTK_SIGNAL_FUNC(on_pagefinish_finish),this);
}
