/* dialogs.h - Common dialog classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifndef _DIALOGS_H
#define _DIALOGS_H

#include <string>

#include <gnome.h>
#include <glade/glade.h>

#include "iexceptions.h"
#include "views.h"

class WidgetOwner
{
public:
	virtual GtkWidget *get_widget(const string &name) const = 0;
};

class DialogOwner
{
public:
	virtual void error(const string &msg) const = 0;
	virtual void warning(const string &msg) const = 0;
	virtual void message(const string &msg) const = 0;
	virtual void flash(const string &msg) const = 0;

	virtual int question_ok_cancel(const string &msg) const = 0;
	virtual int question_yes_no(const string &msg) const = 0;
	virtual int question_yes_no_cancel(const string &msg) const = 0;
};

class GladeWidgetOwner : public WidgetOwner
{
protected:
	GladeXML *xml;
public:
	virtual GtkWidget *get_widget(const string &name) const;
};

class Application : virtual public GladeWidgetOwner, public DialogOwner
{
public:
	virtual void set_title(const string &title) const = 0;
	virtual void up_interface(void) const = 0;

	virtual int execute(const string &command,const bool show,const bool clear,GtkCList* out,GtkCList *err) const = 0;

	virtual View *get_file_view(void) const = 0;
	
	virtual string get_homedir(void) const = 0;
	virtual string get_templdir(void) const = 0;

	virtual void create_file(const string &file,const string &templ,const string &descr) const = 0;
	virtual void create_file(const string &file,const string &orig) const = 0;
};

class Dialog : virtual public GladeWidgetOwner
{
protected:
	GtkWidget *window;
	
	virtual void connect_signals(void)
		{	
			glade_xml_signal_autoconnect(xml);
		};
public:
	Dialog(const string &file,const string &name);
	virtual ~Dialog();
	virtual int run(void);
};

class GtkedDialog : public Dialog
{
public:
	GtkedDialog(const string &file,const string &name) :
		Dialog(file,name)
		{ };
	virtual int run(void);
};

class GnomedDialog : public Dialog
{
public:
	GnomedDialog(const string &file,const string &name) :
		Dialog(file,name)
		{ };
	virtual int run(void);
};

class DruidDialog : public GtkedDialog
{
protected:
	//callbacks
	static void on_druid_cancel(GnomeDruid *druid,DruidDialog *dia);
	static void on_pagefinish_finish(GnomeDruidPage *druid_page,GnomeDruid *druid,DruidDialog *dia);

	virtual void connect_signals(void);
public:
	DruidDialog(const string &file,const string &name) :
		GtkedDialog(file,name)
		{
			connect_signals();
		};
};

#endif
