/* files.cc - File classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#define MAX_BUF_SIZE 128

#include "files.h"

//File
File::File(const string &_name,View *_view,GtkNotebook *_nb,GtkTreeItem *_item)
{
	int pos = _name.rfind('/');
	prefix.assign(_name,0,pos);
	name.assign(_name,pos+1,_name.size()-pos-1);
	fullname = _name;
	
	nb = _nb;
	item = _item;

	view = _view;

	label = GTK_LABEL(gtk_label_new(name.c_str()));
	gtk_notebook_append_page(nb,view->get_widget(),GTK_WIDGET(label));
	int num = gtk_notebook_page_num(nb,view->get_widget());
	gtk_notebook_set_page(nb,num);

	gtk_object_set_data(GTK_OBJECT(gtk_notebook_get_nth_page(nb,num)),"file",this);

	load();
}

File::~File()
{
	int page = gtk_notebook_page_num(nb,view->get_widget());
	
	delete view;
	gtk_notebook_remove_page(nb,page);
}

void File::load(void)
{
	view->load(fullname);
}

void File::save(void)
{
	view->save(fullname);
}

void File::save_as(const string &newname)
{
	view->save(newname);
	
	int pos = newname.rfind('/');
	prefix.assign(newname,0,pos);
	name.assign(newname,pos+1,newname.size()-pos-1);
	fullname = newname;
	
	gtk_label_set_text(label,name.c_str());
}

void File::activate(void)
{
	int page = gtk_notebook_page_num(nb,view->get_widget());
	gtk_notebook_set_page(nb,page);
}
