/* prefs.cc - Preferences classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include <stdlib.h>

#include "util.h"
#include "conffile.h"
#include "titano-dialogs.h"
#include "prefs.h"

bool Preferences::check(void)
{
	return file_exists(file);
}

void Preferences::defaults(void)
{
	if(name.empty())
		name = getenv("USER");
	
	if(mail.empty()){
		char *host = getenv("HOSTNAME");
		mail = name + "@" + host;
	};

	if(defaultdir.empty()){
		defaultdir = parent.homedir + "projects";
		if(!dir_exists(defaultdir))
			dir_create(defaultdir);
	};

	if(!recentmax)
		recentmax = 5;
	
	//prepare editor settings
	if(view.empty()){
		view = "Scintilla";
		sets.spaces = false;
		sets.tab_stop = 8;
		sets.viewws = false;
		sets.vieweol = false;
		sets.indentation = false;
		sets.selcolor = "#FFFFFF";
		sets.selback = "#000000";
	};
}

void Preferences::create(void) throw(EIOError)
{
	dir_create(parent.homedir + ".titano");
	file_create(file);
}

void Preferences::load(void) throw(EIOError)
{
	ConfFile prefs(file);

	prefs.read("General/defdir",defaultdir);
	prefs.read("General/lastproj",lastproj);
	prefs.read("General/recentmax",recentmax);

	prefs.read("User/name",name);
	prefs.read("User/mail",mail);

	prefs.read("Editor/view",view);
	prefs.read("Editor/spaces",sets.spaces);
	prefs.read("Editor/tabstop",sets.tab_stop);
	prefs.read("Editor/viewws",sets.viewws);
	prefs.read("Editor/vieweol",sets.vieweol);
	prefs.read("Editor/indentation",sets.indentation);
	prefs.read("Editor/selcolor",sets.selcolor);
	prefs.read("Editor/selback",sets.selback);

	ConfFile::iterator iter = prefs.get_iterator("Recent");
	string file,foo;
	iter = prefs.iterator_next(iter,file,foo);
	while(iter){
		if(file_exists(file))
			recent.push_front(file);

		iter = prefs.iterator_next(iter,file,foo);
	};

	prefs.drop();
}

void Preferences::save(void) throw(EIOError)
{
	ConfFile prefs(file);
	
	prefs.write("General/defdir",defaultdir);
	if(parent.project){
		lastproj = parent.project->get_filename();
		prefs.write("General/lastproj",lastproj);
	}else{
		if(!lastproj.empty())
			prefs.write("General/lastproj",lastproj);
		else
			prefs.write("General/lastproj","");
	};

	recent_list::iterator iter = recent.begin();
	prefs.clear_section("Recent");
	while(iter != recent.end())
		prefs.write("Recent/" + (*iter++),"1");

	prefs.write("Editor/view",view);
	prefs.write("Editor/spaces",sets.spaces);
	prefs.write("Editor/tabstop",sets.tab_stop);
	prefs.write("Editor/viewws",sets.viewws);
	prefs.write("Editor/vieweol",sets.vieweol);
	prefs.write("Editor/indentation",sets.indentation);
	prefs.write("Editor/selcolor",sets.selcolor);
	prefs.write("Editor/selback",sets.selback);
	
	prefs.write("User/name",name);
	prefs.write("User/mail",mail);
	
	prefs.drop();
}

void Preferences::setup(void)
{
	PrefsDialog dia(*this);

	dia.run();
}




