/* titano-app.cc - Main Titano class -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <fstream.h>
#include <errno.h>

#include "titano-dialogs.h"
#include "conffile.h"
#include "util.h"
#include "prefs.h"
#include "titano-app.h"

TitanoApp::~TitanoApp()
{
	delete prefs;

	if(project)             //should be cleaned earlier, but nevertheless
		delete project;
}

void TitanoApp::connect_signals(void)
{
	gtk_signal_connect(GTK_OBJECT(window),"delete_event",GTK_SIGNAL_FUNC(on_exitw_activate),this);

	//menus
	//#File#
	gtk_signal_connect(GTK_OBJECT(get_widget("new_proj")),"activate",GTK_SIGNAL_FUNC(on_new_proj_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("open_proj")),"activate",GTK_SIGNAL_FUNC(on_open_proj_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_proj")),"activate",GTK_SIGNAL_FUNC(on_save_proj_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_proj_as")),"activate",GTK_SIGNAL_FUNC(on_save_proj_as_activate),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("save_file")),"activate",GTK_SIGNAL_FUNC(on_save_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_all_files")),"activate",GTK_SIGNAL_FUNC(on_save_all_files_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_file_as")),"activate",GTK_SIGNAL_FUNC(on_save_file_as_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("close_file")),"activate",GTK_SIGNAL_FUNC(on_close_file_activate),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("exit")),"activate",GTK_SIGNAL_FUNC(on_exit_activate),this);

	//#Edit#
	gtk_signal_connect(GTK_OBJECT(get_widget("undo")),"activate",GTK_SIGNAL_FUNC(on_undo_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("redo")),"activate",GTK_SIGNAL_FUNC(on_redo_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("cut")),"activate",GTK_SIGNAL_FUNC(on_cut_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("copy")),"activate",GTK_SIGNAL_FUNC(on_copy_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("paste")),"activate",GTK_SIGNAL_FUNC(on_paste_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("clear")),"activate",GTK_SIGNAL_FUNC(on_clear_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("changelog")),"activate",GTK_SIGNAL_FUNC(on_changelog_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("date&time")),"activate",GTK_SIGNAL_FUNC(on_date_time_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("name&mail")),"activate",GTK_SIGNAL_FUNC(on_name_mail_activate),this);

	//#Search#
	gtk_signal_connect(GTK_OBJECT(get_widget("goto_line")),"activate",GTK_SIGNAL_FUNC(on_goto_line_activate),this);

	//#Project#
	gtk_signal_connect(GTK_OBJECT(get_widget("new_file")),"activate",GTK_SIGNAL_FUNC(on_new_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("add_file")),"activate",GTK_SIGNAL_FUNC(on_add_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("remove_file")),"activate",GTK_SIGNAL_FUNC(on_remove_file_activate),this);

	gtk_signal_connect(GTK_OBJECT(get_widget("distribution")),"activate",GTK_SIGNAL_FUNC(on_distribution_activate),this);
	
	//#Compile#
	gtk_signal_connect(GTK_OBJECT(get_widget("automake")),"activate",GTK_SIGNAL_FUNC(on_autoconf_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("configure")),"activate",GTK_SIGNAL_FUNC(on_configure_activate),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("make")),"activate",GTK_SIGNAL_FUNC(on_make_activate),this);

	//#Preferences#
	gtk_signal_connect(GTK_OBJECT(get_widget("preferences")),"activate",GTK_SIGNAL_FUNC(on_preferences_activate),this);
	
	//#Help#
	gtk_signal_connect(GTK_OBJECT(get_widget("about")),"activate",GTK_SIGNAL_FUNC(on_about_activate),this);

	//toolbar
	gtk_signal_connect(GTK_OBJECT(get_widget("open_proj_btn")),"clicked",GTK_SIGNAL_FUNC(on_open_proj_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_proj_btn")),"clicked",GTK_SIGNAL_FUNC(on_save_proj_activate),this);

	gtk_signal_connect(GTK_OBJECT(get_widget("new_file_btn")),"clicked",GTK_SIGNAL_FUNC(on_new_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_file_btn")),"clicked",GTK_SIGNAL_FUNC(on_save_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("save_all_files_btn")),"clicked",GTK_SIGNAL_FUNC(on_save_all_files_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("close_file_btn")),"clicked",GTK_SIGNAL_FUNC(on_close_file_activate),this);

	gtk_signal_connect(GTK_OBJECT(get_widget("addf_btn")),"clicked",GTK_SIGNAL_FUNC(on_add_file_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("remf_btn")),"clicked",GTK_SIGNAL_FUNC(on_remove_file_activate),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("autogen_btn")),"clicked",GTK_SIGNAL_FUNC(on_autoconf_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("conf_btn")),"clicked",GTK_SIGNAL_FUNC(on_configure_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("make_btn")),"clicked",GTK_SIGNAL_FUNC(on_make_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("dist_btn")),"clicked",GTK_SIGNAL_FUNC(on_distribution_activate),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("cut_btn")),"clicked",GTK_SIGNAL_FUNC(on_cut_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("copy_btn")),"clicked",GTK_SIGNAL_FUNC(on_copy_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("paste_btn")),"clicked",GTK_SIGNAL_FUNC(on_paste_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("undo_btn")),"clicked",GTK_SIGNAL_FUNC(on_undo_activate),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("redo_btn")),"clicked",GTK_SIGNAL_FUNC(on_redo_activate),this);
}

void TitanoApp::up_interface(void) const
{
	if(project){
		gtk_widget_set_sensitive(get_widget("save_proj"),TRUE);
		gtk_widget_set_sensitive(get_widget("save_proj_as"),TRUE);
		
		gtk_widget_show(get_widget("project"));
		gtk_widget_show(get_widget("compile"));
		gtk_widget_set_sensitive(get_widget("maketoolbar"),TRUE);
		
		gtk_widget_set_sensitive(get_widget("save_proj_btn"),TRUE);
		gtk_widget_set_sensitive(get_widget("addf_btn"),TRUE);
		gtk_widget_set_sensitive(get_widget("remf_btn"),TRUE);
		gtk_widget_set_sensitive(get_widget("new_file_btn"),TRUE);
		if(project->has_file()){
			gtk_widget_set_sensitive(get_widget("save_file"),TRUE);
			gtk_widget_set_sensitive(get_widget("save_file_as"),TRUE);
			gtk_widget_set_sensitive(get_widget("save_all_files"),TRUE);
			gtk_widget_set_sensitive(get_widget("close_file"),TRUE);

			gtk_widget_set_sensitive(get_widget("save_file_btn"),TRUE);
			gtk_widget_set_sensitive(get_widget("save_all_files_btn"),TRUE);
			gtk_widget_set_sensitive(get_widget("close_file_btn"),TRUE);
			
			gtk_widget_set_sensitive(get_widget("edittoolbar"),TRUE);
			
			gtk_widget_show(get_widget("edit"));
			gtk_widget_show(get_widget("search"));
		}else{
			gtk_widget_set_sensitive(get_widget("save_file"),FALSE);
			gtk_widget_set_sensitive(get_widget("save_file_as"),FALSE);
			gtk_widget_set_sensitive(get_widget("save_all_files"),FALSE);
			gtk_widget_set_sensitive(get_widget("close_file"),FALSE);

			gtk_widget_set_sensitive(get_widget("save_file_btn"),FALSE);
			gtk_widget_set_sensitive(get_widget("save_all_files_btn"),FALSE);
			gtk_widget_set_sensitive(get_widget("close_file_btn"),FALSE);
			
			gtk_widget_set_sensitive(get_widget("edittoolbar"),FALSE);
			
			gtk_widget_hide(get_widget("edit"));
			gtk_widget_hide(get_widget("search"));
		};
	}else{
		gtk_widget_set_sensitive(get_widget("save_proj"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_proj_as"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_file"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_file_as"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_all_files"),FALSE);
		gtk_widget_set_sensitive(get_widget("close_file"),FALSE);
		
		gtk_widget_hide(get_widget("edit"));
		gtk_widget_hide(get_widget("search"));
		gtk_widget_hide(get_widget("project"));
		gtk_widget_hide(get_widget("compile"));
		gtk_widget_set_sensitive(get_widget("maketoolbar"),FALSE);
		
		gtk_widget_set_sensitive(get_widget("save_proj_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("addf_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("remf_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("new_file_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_file_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("save_all_files_btn"),FALSE);
		gtk_widget_set_sensitive(get_widget("close_file_btn"),FALSE);
		
		gtk_widget_set_sensitive(get_widget("edittoolbar"),FALSE);
	};
}

int TitanoApp::run(void)
{
	prefs = new Preferences(*this);
	
	if(!prefs->lastproj.empty())
		open_project(prefs->lastproj);

	up_interface();
	update_recent();
	gtk_widget_show(window);
	
	gtk_main();
}

void TitanoApp::error(const string &msg) const
{
	gnome_app_error(GNOME_APP(window),msg.c_str());
}

void TitanoApp::warning(const string &msg) const
{
	gnome_app_warning(GNOME_APP(window),msg.c_str());
}

void TitanoApp::message(const string &msg) const
{
	gnome_app_message(GNOME_APP(window),msg.c_str());
}

void TitanoApp::flash(const string &msg) const
{
	gnome_app_flash(GNOME_APP(window),msg.c_str());
}

int TitanoApp::question_ok_cancel(const string &msg) const
{
	GtkWidget *dia = gnome_app_ok_cancel_modal(GNOME_APP(window),msg.c_str(),null_cb,NULL);
	return gnome_dialog_run_and_close(GNOME_DIALOG(dia));
}

int TitanoApp::question_yes_no(const string &msg) const
{
	GtkWidget *dia = gnome_app_question_modal(GNOME_APP(window),msg.c_str(),null_cb,NULL);
	return gnome_dialog_run_and_close(GNOME_DIALOG(dia));
}

int TitanoApp::question_yes_no_cancel(const string &msg) const
{
	YNC_Dialog dia(msg);
	return dia.run();
}

void TitanoApp::set_title(const string &title) const
{
	string tmptitle = name + " " + version + title;
	gtk_window_set_title(GTK_WINDOW(window),tmptitle.c_str());
}

View *TitanoApp::get_file_view(void) const
{
	View *view = NULL;
	
	if(prefs->view == "GtkEditor")
		view = new GtkEditorView;
	else
		view = new ScintillaView;
	
	view->set_settings(prefs->sets);
	return view;
}

void TitanoApp::reapply_prefs(void) const
{
	if(project){
		files_list files = project->get_files();
		files_list::iterator iter = files.begin();

		while(iter != files.end()){
			if(project->file_is_opened(*iter))
				(*project)[*iter]->get_view().set_settings(prefs->sets);
			iter++;
		};
	};
}

void TitanoApp::input_out(void *_app,int file,GdkInputCondition cond)
{
	char output[384] = {'\0',};
	static char acc[384] = {'\0',};
	int stat,code;
	char *line[1];
	TitanoApp *app = (TitanoApp *)_app;
	
	if(!read(file,output,sizeof(output))){
		gdk_input_remove(app->id_out);
		
		if(waitpid(app->pid,&stat,0) == -1){
			if(errno != EINTR)
				code = -1;
		}else{
			code = stat;
		};

		app->pid = code;
		sprintf(output,"Exit status: %d",code);
		line[0] = output;
		gtk_clist_append(app->out_list,line);
		app->id_out = 0;
		app->out_list = NULL;
		acc[0] = '\0';
	}else{
		strcat(acc,output);
		int len =strlen(acc);
		char *pos = acc, *start = acc;
		while(char *pos = strchr(start,'\n')){
			*pos = '\0';
			line[0] = start;
			gtk_clist_append(GTK_CLIST(app->out_list),line);
			start = pos+1;
		};
		if(start-acc < len)	
			strcpy(acc,start);
		else
			acc[0] = '\0';
	};
}

void TitanoApp::input_err(void *_app,int file,GdkInputCondition cond)
{
	char output[384] = {'\0',};
	char acc[384] = {'\0',};
	char *line[1];
	TitanoApp *app = (TitanoApp *)_app;

	if(!read(file,output,sizeof(output))){
		gdk_input_remove(app->id_err);
		app->id_err = 0;
		app->err_list = NULL;
		acc[0] = '\0';
	}else{
		strcat(acc,output);
		int len =strlen(acc);
		char *pos = acc, *start = acc;
		while(char *pos = strchr(start,'\n')){
			*pos = '\0';
			line[0] = start;
			gtk_clist_append(GTK_CLIST(app->err_list),line);
			start = pos+1;
		};
		if(start-acc < len)	
			strcpy(acc,start);
		else
			acc[0] = '\0';
	};
}

int TitanoApp::execute(const string &command,const bool show,const bool clear,GtkCList* out,GtkCList *err) const
{
	char **args = g_strsplit(command.c_str()," ",127);
	int epipe_o[2], epipe_e[2];
	int code;
	char *line[1];

	if(!id_out){
		out_list = out;
		err_list = err;
		
		pipe(epipe_o);
		pipe(epipe_e);
		if((pid = fork()) == -1){
			g_strfreev(args);
			throw Exception("fork() failed.");
		};
		if(pid != 0){
			//parent
			close(epipe_o[1]);
			close(epipe_e[1]);
			if(clear){
				gtk_clist_clear(out);
				gtk_clist_clear(err);
			};
			if(show){
				line[0] = (char *)(command.c_str());
				gtk_clist_append(out,line);
			};
			id_out = gdk_input_add(epipe_o[0],GDK_INPUT_READ,input_out,(void *)this);
			id_err = gdk_input_add(epipe_e[0],GDK_INPUT_READ,input_err,(void *)this);
		}else{
			//child
			close(epipe_o[0]);
			close(epipe_e[0]);
			dup2(epipe_o[1],1);
			dup2(epipe_e[1],2);
			sleep(1);
			execvp(args[0],args);
			printf("Execution failed.");
			_exit(1);
		};
	};
	g_strfreev(args);

	while(id_out)
		g_main_iteration(true);
	code = pid;
	pid = 0;
	
	return code;
}

string TitanoApp::get_full_time(void)
{
	char out_time[128], *locale;
	time_t ctime;
	time(&ctime);
	
	struct tm *t = localtime(&ctime);
	locale = setlocale(LC_TIME,NULL);
	setlocale(LC_TIME,"C");
	strftime(out_time,sizeof(out_time),"%a %b %d %X %Z %Y",t);
	setlocale(LC_TIME,locale);
	
	return out_time;
}	

void TitanoApp::open_project(const string &filename)
{
	if(filename.rfind('.') == filename.npos || filename.compare(".titano",filename.rfind('.'),7)){
		throw Exception("File " + filename + " is not a Titano project.");	       
	};
	
	ConfFile proj(filename);
	string type;
	
	proj.read("General/type",type);
	if(type.substr(0,1) == "c"){
		project = new CppProject(filename,*this);				
	};
}

bool TitanoApp::close_project(void)
{
	if(project){
		if(project->get_modified()){
			switch(question_yes_no_cancel("Current project was modified. Save?")){
			case 0:project->save();
				break;
			case 2:return false;
			};
		};

		files_list files = project->get_files();
		files_list::iterator iter = files.begin();

		while(iter != files.end()){
			if(project->file_is_opened(*iter))
				if((*project)[*iter]->modified()){
					switch(question_yes_no_cancel("File " + *iter + " was modified. Save?")){
						case 0:(*project)[*iter]->save();
							break;
						case 2:return false;
					};
				};
			iter++;
		};

		//Update recently open projects list
		if(find(prefs->recent.begin(),prefs->recent.end(),project->get_filename()) == prefs->recent.end())
			prefs->recent.push_front(project->get_filename());
		while(prefs->recent.size() > prefs->recentmax)
			prefs->recent.pop_back();
		update_recent();
	};

	delete project;
	project = NULL;

	set_title("");
	
	return true;
}

void TitanoApp::update_recent(void)
{
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(get_widget("open_recent")));
	GtkMenu *menu = GTK_MENU(gtk_menu_new());

	GtkWidget *item;
	recent_list::iterator iter = prefs->recent.begin();
	while(iter != prefs->recent.end()){
		item = gtk_menu_item_new_with_label((*iter++).c_str());
		gtk_signal_connect(GTK_OBJECT(item),"activate",GTK_SIGNAL_FUNC(on_recent_activate),this);
		gtk_widget_show(item);
		gtk_menu_append(menu,item);
	};

	gtk_widget_show(GTK_WIDGET(menu));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(get_widget("open_recent")),GTK_WIDGET(menu));
}

void TitanoApp::create_file(const string &file,const string &templ,const string &descr) const
{
	if(file_exists(file)){
		if(question_yes_no("File " + file + " already exists. Overwrite?"))
			throw Exception("File " + file + " already exists.");
	};
			
	ofstream items((homedir + ".titano/.items").c_str());
	if(!items){
		throw Exception("Couldn't create file ~/.titano/.items.");
	};
			
	//fill data
	items << "create" << endl
	      << file << endl
	      << templ << endl
	      << "DESCR" << endl << descr << endl
	      << "AUTHOR" << endl << prefs->name << endl
	      << "MAIL" << endl << prefs->mail << endl;
	
	if(execute(string("perl ") + PACKAGE_DATA_DIR + "/" +PACKAGE + "/scripts/file.pl",
			false,true,GTK_CLIST(get_widget("compile_list")),GTK_CLIST(get_widget("compile_list")))){
		throw Exception("File creation failed.");
	};
}

void TitanoApp::create_file(const string &file,const string &orig) const
{
	if(file_exists(file)){
		if(question_yes_no("File " + file + " already exists. Overwrite?"))
			throw Exception("File " + file + " already exists.");
	};
			
	ofstream items((homedir + ".titano/.items").c_str());
	if(!items){
		throw Exception("Couldn't create file ~/.titano/.items.");
	};
			
	//fill data
	items << "copy" << endl
	      << file << endl
	      << orig << endl
	      << "AUTHOR" << endl << prefs->name << endl
	      << "MAIL" << endl << prefs->mail << endl;
	
	if(execute(string("perl ") + PACKAGE_DATA_DIR + "/" +PACKAGE + "/scripts/file.pl",
			false,true,GTK_CLIST(get_widget("compile_list")),GTK_CLIST(get_widget("compile_list")))){
		throw Exception("File creation failed.");
	};
}

//callbacks
void TitanoApp::on_recent_activate(GtkMenuItem *item,TitanoApp *app)
{
	string name = GTK_LABEL(item->item.bin.child)->label;

	app->close_project();

	try{
		app->open_project(name);
		app->flash("Project loaded...");
	}catch(Exception &e){
		app->error(e);
	};
}

void TitanoApp::on_about_activate(GtkMenuItem *item,TitanoApp *app)
{
	GnomedDialog dia("titano","about");
	dia.run();
}


int TitanoApp::on_exitw_activate(GtkWidget *window,GdkEventAny *ev,TitanoApp *app)
{
	app->prefs->save();

	if(!app->close_project())
		return 1;

	gtk_main_quit();
	return 0;
}

void TitanoApp::on_exit_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->prefs->save();
	
	if(!app->close_project())
		return;

	gtk_main_quit();
}

void TitanoApp::on_new_proj_activate(GtkMenuItem *item,TitanoApp *app)
{
	if(!app->close_project())
		return;

	NewProjectDialog dia("Project",app->prefs->defaultdir);

	if(dia.run()){
		try{
			string filename = dia.get_dir() + "/" + dia.get_name() + "/" + dia.get_name() + ".titano";

			if(file_exists(filename)){
				if(app->question_yes_no("Project " + filename + " already exists. Overwrite?"))
					return;
			};
			
			ofstream items((app->homedir + ".titano/.items").c_str());
			if(!items){
				throw Exception("Couldn't create file ~/.titano/.items.");
			};
			
			//fill data
			bool correct_name = true;
			
			if(dia.generate_am())
				items << "install";
			else{
				items << "import";
				correct_name = false;
			};
			items << endl << dia.get_dir() << endl
			      << dia.get_name() << endl
			      << string(PACKAGE_DATA_DIR) + "/" +PACKAGE + "/templates/" + dia.get_type() + ".tar.gz" << endl
			      << string(PACKAGE_DATA_DIR) + "/" +PACKAGE + "/scripts" << endl
			      << "VERSION" << endl << dia.get_ver() << endl
			      << "USER" << endl << app->prefs->name << endl
			      << "MAIL" << endl << app->prefs->mail << endl;
			    
			if(app->execute(string("perl ") + PACKAGE_DATA_DIR + "/" +PACKAGE + "/scripts/project.pl",
					false,true,GTK_CLIST(app->get_widget("compile_list")),GTK_CLIST(app->get_widget("compile_list")))){
				throw Exception("Project creation failed.");
			};

			if(!correct_name){
				FILE *f = popen(("ls " + dia.get_dir() + "| grep titano").c_str(),"r");
				char buf[128];
				
				fgets(buf,sizeof(buf),f);
				(*strrchr(buf,'\n')) = '\0';
				filename = dia.get_dir() + buf;
				pclose(f);
			}else{		//we did install
				app->flash("Autoconfiguring project...");
				chdir((dia.get_dir() + "/" + dia.get_name()).c_str());
				app->execute("./autogen.sh",true,false,GTK_CLIST(app->get_widget("compile_list")),GTK_CLIST(app->get_widget("compile_list")));
			};
			app->open_project(filename);

			app->flash("Project loaded.");

			
		}catch(Exception &e){
			app->error(e);
		};
	};

	app->up_interface();
}

void TitanoApp::on_open_proj_activate(GtkMenuItem *item,TitanoApp *app)
{
	if(!app->close_project())
		return;
	
	FileDialog dia("Open project...",app->homedir);

	if(dia.run()){
		try{
			app->open_project(dia.get_name());

			app->flash("Project loaded.");
		}catch(Exception &e){
			app->error(e);
		};
	};

	app->up_interface();
}

void TitanoApp::on_save_proj_activate(GtkMenuItem *item,TitanoApp *app)
{
	try{
		app->project->save();

		app->flash("Project saved.");
	}catch(Exception &e){
		app->error(e);
	};
}

void TitanoApp::on_save_proj_as_activate(GtkMenuItem *item,TitanoApp *app)
{
	FileDialog dia("Save project as...",app->homedir);

	if(dia.run()){
		try{
			string filename = dia.get_name();
			
			if(file_exists(filename)){
				if(app->question_yes_no("Project " + filename + " already exists. Overwrite?"))
					return;				
			};

			app->project->save_as(filename);
		}catch(Exception &e){
			app->error(e);
		};
	};

	app->flash("Project saved.");
}

void TitanoApp::on_save_file_activate(GtkMenuItem *item,TitanoApp *app)
{
	try{
		app->project->get_file().save();

		app->flash("File saved.");
	}catch(Exception &e){
		app->error(e);
	};
}

void TitanoApp::on_save_file_as_activate(GtkMenuItem *item,TitanoApp *app)
{
	FileDialog dia("Save file as...",app->project->get_dir() + "/src/");

	if(dia.run()){
		try{
			string filename = dia.get_name();
			
			if(file_exists(filename)){
				if(app->question_yes_no("File " + filename + " already exists. Overwrite?"))
					return;				
			};

			app->project->save_file_as(filename);
		}catch(Exception &e){
			app->error(e);
		};
	};

	app->flash("File saved.");
}

void TitanoApp::on_save_all_files_activate(GtkMenuItem *item,TitanoApp *app)
{
	try{
		app->project->save_all_files();

		app->flash("All files were saved.");
	}catch(Exception &e){
		app->error(e);
	};
}

void TitanoApp::on_close_file_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->close_file();

	app->up_interface();

	app->flash("File closed.");
}

void TitanoApp::on_new_file_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->new_file();

	app->up_interface();

	app->flash("File created.");
}
       
void TitanoApp::on_add_file_activate(GtkMenuItem *item,TitanoApp *app)
{
	FileDialog dia("Add file...",app->project->get_dir() + "/src/");

	if(dia.run()){
		try{
			string filename = dia.get_name();

			if(app->project->file_in_project(filename)){
				app->error("File " + filename + " already in project. It would not be added.");
				return;
			};
			
			int pos = filename.rfind('/');
			string newfilename = app->project->get_dir() + "/src" + filename.substr(pos,filename.size()-pos);

			if(!file_exists(filename))
				file_create(newfilename);
			else
				if(filename != newfilename)
					app->create_file(newfilename,filename);

			app->project->add_file(newfilename);
			app->project->open_file(newfilename);

			app->flash("File added.");
		}catch(Exception &e){
			app->error(e);
		};
	};
}

void TitanoApp::on_remove_file_activate(GtkMenuItem *item,TitanoApp *app)
{
	RemoveFileDialog dia(app->project->get_files());

	if(dia.run()){
		try{
			string filename = dia.get_name();

			if(filename != "")
				app->project->remove_file(filename);

			app->flash("File removed.");
		}catch(Exception &e){
			app->error(e);
		};
	};
}

void TitanoApp::on_autoconf_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->autogen();
}

void TitanoApp::on_configure_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->configure();
}

void TitanoApp::on_distribution_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->make_dist();
}

void TitanoApp::on_make_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->make();
}

void TitanoApp::on_undo_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().undo();
}

void TitanoApp::on_redo_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().redo();
}

void TitanoApp::on_cut_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().cut();
}

void TitanoApp::on_copy_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().copy();
}

void TitanoApp::on_paste_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().paste();
}

void TitanoApp::on_clear_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().clear();
}

void TitanoApp::on_changelog_activate(GtkMenuItem *item,TitanoApp *app)
{
	char out[128];

	sprintf(out,"%s %s <%s>",app->get_full_time().c_str(),app->prefs->name.c_str(),app->prefs->mail.c_str());
	
	app->project->get_file().get_view().insert(out);
}

void TitanoApp::on_date_time_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->project->get_file().get_view().insert(app->get_full_time());
}

void TitanoApp::on_name_mail_activate(GtkMenuItem *item,TitanoApp *app)
{
	char out[128];
	
	sprintf(out,"%s <%s>",app->prefs->name.c_str(),app->prefs->mail.c_str());
	
	app->project->get_file().get_view().insert(out);
}	

void TitanoApp::on_preferences_activate(GtkMenuItem *item,TitanoApp *app)
{
	app->prefs->setup();
}

void TitanoApp::on_goto_line_activate(GtkMenuItem *item,TitanoApp *app)
{
	GotoLineDialog dia;

	if(dia.run() == 0){
		app->project->get_file().get_view().set_pos(dia.get_line(),1);
	};
}