/* titano-app.h - Main Titano class -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifndef _TITANO_APP_H
#define _TITANO_APP_H

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdlib.h>

#include "dialogs.h"
#include "projects.h"

class Preferences;

class TitanoApp : public Dialog, public Application
{
private:
	string name,version;

	string homedir,templdir;
	
	Preferences *prefs;
	
	Project *project;

	mutable int pid,id_out,id_err;
	mutable GtkCList *out_list, *err_list;

	virtual void connect_signals(void);
	
	string get_full_time(void);
	
	virtual bool close_project(void);
	virtual void open_project(const string &filename);

	virtual void update_recent(void);

	//callbacks
	static void null_cb(int,void *) { };

	static void input_out(void *_app,int file,GdkInputCondition cond);
	static void input_err(void *_app,int file,GdkInputCondition cond);

	static void on_recent_activate(GtkMenuItem *item,TitanoApp *app);
	
	static void on_about_activate(GtkMenuItem *item,TitanoApp *app);
	static int on_exitw_activate(GtkWidget *window,GdkEventAny *ev,TitanoApp *app);
	static void on_exit_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_new_proj_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_open_proj_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_save_proj_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_save_proj_as_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_save_file_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_save_file_as_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_save_all_files_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_close_file_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_new_file_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_add_file_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_remove_file_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_autoconf_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_configure_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_distribution_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_make_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_undo_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_redo_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_cut_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_copy_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_paste_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_clear_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_changelog_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_date_time_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_name_mail_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_preferences_activate(GtkMenuItem *item,TitanoApp *app);
	static void on_goto_line_activate(GtkMenuItem *item,TitanoApp *app);
public:
	TitanoApp(int argc,char *argv[]) :
		Dialog("titano","mainapp")
	{
		name = "Titano";
		version = VERSION;

		homedir = string(getenv("HOME")) + "/";
		templdir = string(PACKAGE_DATA_DIR) + "/" +PACKAGE + "/templates/";
		
		project = NULL;
		prefs = NULL;

		id_out = id_err = 0;
		out_list = err_list = NULL;

		connect_signals();

		set_title("");
	};
	virtual ~TitanoApp();

	virtual string get_homedir(void) const
	{
		return homedir;
	};
	virtual string get_templdir(void) const
	{
		return templdir;
	};

	virtual void up_interface(void) const;
	
	virtual int run(void);

	virtual void error(const string &msg) const;
	virtual void warning(const string &msg) const;
	virtual void message(const string &msg) const;
	virtual void flash(const string &msg) const;

	virtual int question_ok_cancel(const string &msg) const;
	virtual int question_yes_no(const string &msg) const;
	virtual int question_yes_no_cancel(const string &msg) const;

	virtual void set_title(const string &title) const;
	virtual int execute(const string &command,const bool show,const bool clear,GtkCList* out,GtkCList *err) const;

	virtual View *get_file_view(void) const;

	virtual void reapply_prefs(void) const;

	virtual void create_file(const string &file,const string &templ,const string &descr) const;
	virtual void create_file(const string &file,const string &orig) const;
	
	friend class Preferences;
};

#endif
