/* titano-dialogs.cc - Titano specific dialog classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include "titano-dialogs.h"

//NewProjectDialog
void NewProjectDialog::connect_signals(void)
{
	GtkedDialog::connect_signals();
	
	gtk_signal_connect(GTK_OBJECT(get_widget("name")),"changed",GTK_SIGNAL_FUNC(on_name_changed),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("createproj")),"toggled",GTK_SIGNAL_FUNC(on_toggled),this);
}

string NewProjectDialog::get_name(void)
{
	return gtk_entry_get_text(GTK_ENTRY(get_widget("name")));
}

string NewProjectDialog::get_dir(void)
{
	if(generate_am())
		return gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(get_widget("direntry")),false);
	else
		return gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(get_widget("topdirentry")),false);
}

string NewProjectDialog::get_ver(void)
{
	return gtk_entry_get_text(GTK_ENTRY(get_widget("verentry")));
}

string NewProjectDialog::get_type(void)
{
	static const char *names[][3] = {
		{"cconsole","cgtk","cgnome"},
		{"cppconsole","cppgtk","cppgnome"}
	};
	
	int page = gtk_notebook_get_current_page(GTK_NOTEBOOK(get_widget("ptypenb")));
	GnomeIconList *list;
	int icon;
	switch(page){
	case 0:
		list = GNOME_ICON_LIST(get_widget("clist"));
		icon = int(list->selection->data);
		break;
	case 1:
		list = GNOME_ICON_LIST(get_widget("cpplist"));
		icon = int(list->selection->data);
		break;	
	};
	
	return names[page][icon];
}

bool NewProjectDialog::generate_am(void)
{
	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("createproj")));
}

//callbacks
void NewProjectDialog::on_name_changed(GtkEntry *entry,NewProjectDialog *dia)
{
	if(!strlen(gtk_entry_get_text(entry))){
		gnome_druid_set_buttons_sensitive(GNOME_DRUID(dia->get_widget("druid")),true,false,true);
	}else{
		gnome_druid_set_buttons_sensitive(GNOME_DRUID(dia->get_widget("druid")),true,true,true);
	};
}

void NewProjectDialog::on_toggled(GtkRadioButton *btn,NewProjectDialog *dia)
{
	bool foo = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn));

	gtk_widget_set_sensitive(dia->get_widget("createtable"),foo);
	gtk_widget_set_sensitive(dia->get_widget("importtable"),!foo);
}

//RemoveFileDialog
void RemoveFileDialog::connect_signals(void)
{
	gtk_signal_connect(GTK_OBJECT(get_widget("okbutton")),"clicked",GTK_SIGNAL_FUNC(on_ok),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("cancelbutton")),"clicked",GTK_SIGNAL_FUNC(on_cancel),this);
}

string RemoveFileDialog::get_name(void)
{
	if(!flist->selection)
		return "";

	char *name;
	GtkLabel *lbl = GTK_LABEL(GTK_LIST_ITEM(flist->selection->data)->item.bin.child);
	gtk_label_get(lbl,&name);
	
	return name;
}

//PrefsDialog
void PrefsDialog::connect_signals(void)
{
	GtkedDialog::connect_signals();

	gtk_signal_connect(GTK_OBJECT(window),"close",GTK_SIGNAL_FUNC(on_destroy),this);
	gtk_signal_connect(GTK_OBJECT(window),"apply",GTK_SIGNAL_FUNC(on_apply),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("recentspin")),"changed",GTK_SIGNAL_FUNC(on_change),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("nameentry")),"changed",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("mailentry")),"changed",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("projdircentry")),"changed",GTK_SIGNAL_FUNC(on_change),this);
	
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(get_widget("viewnamecombo"))->entry),"changed",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("tabstopspin")),"changed",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("usespacescb")),"toggled",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("wscb")),"toggled",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("eolncb")),"toggled",GTK_SIGNAL_FUNC(on_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("indentcb")),"toggled",GTK_SIGNAL_FUNC(on_change),this);
	
	gtk_signal_connect(GTK_OBJECT(get_widget("selcolor")),"color_set",GTK_SIGNAL_FUNC(on_color_change),this);
	gtk_signal_connect(GTK_OBJECT(get_widget("backcolor")),"color_set",GTK_SIGNAL_FUNC(on_color_change),this);
};

void PrefsDialog::on_destroy(GtkWidget *widget,PrefsDialog *dia)
{
	gtk_main_quit();
}
	
void PrefsDialog::on_change(GtkWidget *widget,PrefsDialog *dia)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(dia->window));
};

void PrefsDialog::on_color_change(GtkWidget *widget,guint r,guint g,guint b,guint a,PrefsDialog *dia)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(dia->window));
};

void PrefsDialog::on_apply(GnomePropertyBox *box,int page,PrefsDialog *dia)
{
	//General
	dia->parent.name = gtk_entry_get_text(GTK_ENTRY(dia->get_widget("nameentry")));
	dia->parent.mail = gtk_entry_get_text(GTK_ENTRY(dia->get_widget("mailentry")));
	dia->parent.defaultdir = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(dia->get_widget("projdir")),false);
	dia->parent.recentmax = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dia->get_widget("recentspin")));

	//Editor
	dia->parent.view = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dia->get_widget("viewnamecombo"))->entry));
	dia->parent.sets.tab_stop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dia->get_widget("tabstopspin")));
	dia->parent.sets.spaces = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia->get_widget("usespacescb")));
	dia->parent.sets.viewws = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia->get_widget("wscb")));
	dia->parent.sets.vieweol = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia->get_widget("eolncb")));
	dia->parent.sets.indentation = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia->get_widget("indentcb")));

	unsigned char r,g,b,a;
	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(dia->get_widget("selcolor")),&r,&g,&b,&a);
	dia->parent.sets.selcolor = RGB2triplet(r,g,b);
	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(dia->get_widget("backcolor")),&r,&g,&b,&a);
	dia->parent.sets.selback = RGB2triplet(r,g,b);
	
	dia->parent.save();
	if(page == 1)
		dia->parent.parent.reapply_prefs();
		
	gnome_property_box_set_modified(GNOME_PROPERTY_BOX(dia->window),false);
};
