/* titano-dialogs.h - Titano specific dialog classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifndef _TITANO_DIALOGS_H
#define _TITANO_DIALOGS_H

#include <string>
#include <list>

#include "dialogs.h"
#include "prefs.h"
#include "util.h"

#include "../config.h"

class YNC_Dialog : public GnomedDialog
{
public:
	YNC_Dialog(const string &msg) :
		GnomedDialog("titano","yesnocanceldia")
	{
		gtk_label_set_text(GTK_LABEL(get_widget("message")),msg.c_str());
	};
};

class FileDialog
{
private:
	GtkFileSelection *fs;
	int result;

	//callbacks
	static void on_ok(GtkWidget *button,FileDialog *dia)
	{
		dia->result = 1;
		gtk_widget_hide(GTK_WIDGET(dia->fs));
	};
	static void on_cancel(GtkWidget *button,FileDialog *dia)
	{
		gtk_widget_hide(GTK_WIDGET(dia->fs));
	};
public:
	FileDialog(const string &title,const string &name)
	{
		fs = GTK_FILE_SELECTION(gtk_file_selection_new(title.c_str()));
		gtk_file_selection_set_filename(fs,name.c_str());
		result = 0;

		gtk_signal_connect(GTK_OBJECT(fs->ok_button),"clicked",GTK_SIGNAL_FUNC(on_ok),this);
		gtk_signal_connect(GTK_OBJECT(fs->cancel_button),"clicked",GTK_SIGNAL_FUNC(on_cancel),this);
		gtk_signal_connect(GTK_OBJECT(fs),"delete_event",GTK_SIGNAL_FUNC(gtk_main_quit),NULL);
		gtk_signal_connect_after(GTK_OBJECT(fs),"hide",GTK_SIGNAL_FUNC(gtk_main_quit),NULL);
	};
	~FileDialog()
	{
		gtk_widget_destroy(GTK_WIDGET(fs));
	};

	int run(void)
	{
		gtk_widget_show(GTK_WIDGET(fs));
		gtk_main();
		
		return result;
	};

	string get_name(void)
	{
		return gtk_file_selection_get_filename(fs);
	};
};

class NewProjectDialog : public DruidDialog
{
private:
	virtual void connect_signals(void);
	
	//callbacks
	static void on_name_changed(GtkEntry *entry,NewProjectDialog *dia);
	static void on_toggled(GtkRadioButton *btn,NewProjectDialog *dia);
public:
	NewProjectDialog(const string &defname,const string &defdir) :
		DruidDialog("titano","newprojdia")
	{
		gtk_entry_set_text(GTK_ENTRY(get_widget("name")),defname.c_str());
		gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(get_widget("direntry")),defdir.c_str());
		gtk_entry_set_text(GTK_ENTRY(get_widget("dircentry")),defdir.c_str());
		gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(get_widget("topdirentry")),defdir.c_str());
		gtk_entry_set_text(GTK_ENTRY(get_widget("topdircentry")),defdir.c_str());
		
		GnomeIconList *ilist = GNOME_ICON_LIST(get_widget("clist"));
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/console.png","Console");
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/gtk.png","GTK+");
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/gnome.png","GNOME");
		
		ilist = GNOME_ICON_LIST(get_widget("cpplist"));
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/console.png","Console");
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/gtk.png","GTK+");
		gnome_icon_list_append(ilist,PACKAGE_PIXMAPS_DIR "/gnome.png","GNOME");
		
		connect_signals();
	};
	
	string get_name(void);
	string get_dir(void);
	string get_ver(void);
	string get_type(void);
	bool generate_am(void);
};

class RemoveFileDialog : public GtkedDialog
{
private:
	GtkList *flist;
	
	virtual void connect_signals(void);
	
	//callbacks
	static void on_ok(GtkWidget *button,RemoveFileDialog *dia)
	{
		gtk_widget_hide(dia->window);
		gtk_main_quit();
	};
	static void on_cancel(GtkWidget *button,RemoveFileDialog *dia)
	{
		gtk_widget_destroy(dia->window);
		gtk_main_quit();
	};
public:
	RemoveFileDialog(list<string> files) :
		GtkedDialog("titano","removefiledia")
	{
			GList *l = NULL;
			list<string>::iterator iter = files.begin();
			GtkWidget *list_item;
			GtkWidget *label;

			flist = GTK_LIST(get_widget("filelist"));
			
			while(iter != files.end()){
				label = gtk_label_new((*iter).c_str());
				list_item = gtk_list_item_new();
				gtk_container_add(GTK_CONTAINER(list_item),label);
				gtk_widget_show(label);
				gtk_widget_show(list_item);
				
				l = g_list_append(l,list_item);
				iter++;
			};

			gtk_list_append_items(flist,l);
			
			connect_signals();
	};

	string get_name(void);
};

class PrefsDialog : public GtkedDialog
{
private:
	Preferences &parent;
	
	virtual void connect_signals(void);
	
	//callbacks
	static void on_destroy(GtkWidget *widget,PrefsDialog *dia);
	static void on_change(GtkWidget *widget,PrefsDialog *dia);
	static void on_color_change(GtkWidget *widget,guint r,guint g,guint b,guint a,PrefsDialog *dia);
	static void on_apply(GnomePropertyBox *box,int page,PrefsDialog *dia);
public:
	PrefsDialog(Preferences &_parent) : GtkedDialog("titano","prefsdia"), parent(_parent)
	{
		//General
		gtk_entry_set_text(GTK_ENTRY(get_widget("nameentry")),parent.name.c_str());
		gtk_entry_set_text(GTK_ENTRY(get_widget("mailentry")),parent.mail.c_str());

		gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(get_widget("projdir")),parent.defaultdir.c_str());
		gtk_entry_set_text(GTK_ENTRY(get_widget("projdircentry")),parent.defaultdir.c_str());

		gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("recentspin")),parent.recentmax);

		//Editor
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(get_widget("viewnamecombo"))->entry),parent.view.c_str());
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("tabstopspin")),(float)parent.sets.tab_stop);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("usespacescb")),parent.sets.spaces);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("wscb")),parent.sets.viewws);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("eolncb")),parent.sets.vieweol);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("indentcb")),parent.sets.indentation);
		
		unsigned char r,g,b;
		triplet2RGB(parent.sets.selcolor,r,g,b);
		gnome_color_picker_set_i8(GNOME_COLOR_PICKER(get_widget("selcolor")),r,g,b,0);
		triplet2RGB(parent.sets.selback,r,g,b);
		gnome_color_picker_set_i8(GNOME_COLOR_PICKER(get_widget("backcolor")),r,g,b,0);
		
		connect_signals();
	};
};

class GotoLineDialog : public GnomedDialog
{
public:
	GotoLineDialog() : GnomedDialog("titano","gotolinedia")
	{ };

	unsigned long get_line(void)
	{
		return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(get_widget("linespin")));
	};
};

#endif 
