/* util.cc - Some util functions -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <glib.h>
#include "util.h"

bool dir_exists(const string &dir)
{
	char *curdir = NULL;
	bool res;

	curdir = getcwd(curdir,0);
	res = (chdir(dir.c_str()) == 0);
	chdir(curdir);
	g_free(curdir);

	return res;
}

void dir_create(const string &name)
{
	string nname = "mkdir " + name + "/";
	int dir;

	dir =system(nname.c_str());
	if(dir == -1){
		throw EIOError("Dir creation failed.",name);
	};
}

bool file_exists(const string &file)
{
	int f;
	bool res;

	f = open(file.c_str(),O_RDONLY);
	res = (f > 0);
	close(f);
	
	return res;
}

void file_create(const string &file)
{
	FILE *f;

	f = fopen(file.c_str(),"w");
	if(!f){
		throw EIOError("File creation failed.",file);
	};
	fclose(f);
}

string fullname(const string &dir,const string &file)
{
	char *f, *d;
	char dirc[384], filec[384];
	g_snprintf(dirc,sizeof(dirc),"%s",dir.c_str());
	g_snprintf(filec,sizeof(filec),"%s",file.c_str());
	
	d = dirc;
	f = filec;

	while(strstr(f,"..")){
		f += 3;
		*(strrchr(d,'/')) = '\0';
	};
	
	return string(d) + "/" + f;
}

string str_replace(const string &str,const string &rem,const string &ins)
{
	string str1 = str + " ";
	string res1,res;
	gchar **array = g_strsplit(str1.c_str(),rem.c_str(),127);
	
	if(!array[1])
		res = str;
	else{
		res1 = g_strjoinv(ins.c_str(),array);
		if(res1[res1.size()-1] == ' ')
			res.assign(res1,0,res1.size()-1);
		else
			res = res1;	
	};
		
	g_strfreev(array);
	return res;
}

void triplet2RGB(const string &triplet,unsigned char &r,unsigned char &g,unsigned char &b)
{
	int ir,ig,ib;
	sscanf(triplet.c_str(),"#%2X%2X%2X",&ir,&ig,&ib);
	r = ir;
	g = ig;
	b = ib;
}

string RGB2triplet(unsigned char &r,unsigned char &g,unsigned char &b)
{
	char res[7];
	sprintf(res,"#%.2X%.2X%.2X",r,g,b);
	
	return res;
}