/* views.cc - File view classes -*- C++ -*-
 * Copyright (C) 2000 Malenko Dmitry <maldim@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <errno.h>
#include <stdio.h>

#define BUF_MAX_SIZE 328

#include "views.h"

#include "util.h"

#include "scintilla/PropSet.h"
#include "scintilla/Accessor.h"
#include "scintilla/WindowAccessor.h"
#include "scintilla/KeyWords.h"
#include "scintilla/ScintillaWidget.h"
#include "scintilla/Scintilla.h"
#include "scintilla/SciLexer.h"

//View
string View::get_language(const string &file)
{
	string res = "unknown";
	int pos = file.rfind('.');
	
	if(pos != file.npos){
		if(!file.compare(".h",pos,2)||
		   !file.compare(".c",pos,2)||
		   !file.compare(".C",pos,2)||
		   !file.compare(".cc",pos,3)||
		   !file.compare(".cpp",pos,4)||
		   !file.compare(".cxx",pos,4))
		res = "cpp";
	};

	return res;
}

//GtkEditor view
GtkEditorScanner* GtkEditorView::cscanner = NULL;

GtkEditorView::GtkEditorView()
{
	editor = gtk_editor_new();
	buffer = GTK_TEXT_BUFFER(editor);
	GtkTextTag *tag = gtk_editor_get_default_tag(editor);
	GtkWidget *sw;
  
	/* setup the default tag */
	gtk_object_set(GTK_OBJECT(tag),
		       "pixels_above_lines", 0,
		       "pixels_below_lines", 0,
		       "font", "-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*-*",
		       NULL);
	
	window = gtk_viewport_new(NULL, NULL);
	sw = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	view = GTK_TEXT_VIEW(gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(editor)));
	gtk_container_add(GTK_CONTAINER(sw),GTK_WIDGET(view));
	gtk_container_add(GTK_CONTAINER(window),sw);
	
	gtk_widget_show(window);
	gtk_widget_show(sw);
	gtk_widget_show(GTK_WIDGET(view));
}

GtkEditorView::~GtkEditorView()
{
	gtk_widget_destroy(window);
}

void GtkEditorView::setup(const string &file)
{
	GtkTextTag *tag;
	GtkTextIter start, end;

	if(get_language(file) == "cpp"){
		if(!GtkEditorView::cscanner)
			if(!(GtkEditorView::cscanner = gtk_editor_load_scanner(PACKAGE_LIB_DIR "lib/titano/c-scanner.so")))
				return;
		
		gtk_editor_install_scanner(editor,GtkEditorView::cscanner);
	
		tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(editor), "regular");
		

		tag = gtk_editor_create_tag(editor, "keyword");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "blue",
			       "font", "-adobe-courier-bold-r-normal-*-12-*-*-*-*-*-*-*",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "type");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#00d81c",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "include");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#8700bc",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "pre_processor");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#8700bc",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "comment");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#ff0000",
			       "font", "-adobe-courier-medium-i-normal-*-12-*-*-*-*-*-*-*",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "cppcomment");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#ff0000",
			       "font", "-adobe-courier-medium-i-normal-*-12-*-*-*-*-*-*-*",
			       NULL);	       
		
		tag = gtk_editor_create_tag(editor, "string");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#d0b800",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "escape_char");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#4ad822",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "function");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "blue",
			       NULL);
		
		tag = gtk_editor_create_tag(editor, "classname");
		gtk_object_set(GTK_OBJECT(tag),
			       "foreground", "#c000bb",
			       "font", "-adobe-courier-bold-r-normal-*-12-*-*-*-*-*-*-*",
			       NULL);	       
		
		gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),&start,0);
		gtk_text_buffer_get_iter_at_char(GTK_TEXT_BUFFER(editor),&end,get_char_count());
		gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(editor),"regular", &start, &end);
	};
}

GtkWidget *GtkEditorView::get_widget(void)
{
	return window;
}

char *GtkEditorView::get_buffer(void) const
{
	int len = gtk_text_buffer_get_char_count(buffer)-1;
	return gtk_text_buffer_get_text_chars(buffer,0,len,FALSE);
}

bool GtkEditorView::modified(void) const
{
	return gtk_text_buffer_modified(buffer);
}

void GtkEditorView::load(const string &name)
{
	char buf[BUF_MAX_SIZE];
	FILE *f = fopen(name.c_str(),"r");

	setup(name);

	if(!f)
		throw EIOError("Open file for load failed.",name,errno);

	clear();
	
	while(fgets(buf,BUF_MAX_SIZE,f)){
		gtk_text_buffer_insert_at_char(buffer,get_char_count()-1,buf,strlen(buf));
	};

	fclose(f);
};

void GtkEditorView::save(const string &name) const
{
	char *buf;
	FILE *f = fopen (name.c_str(), "w");
	
	if(!f)
		throw EIOError("Open file for save failed.",name,errno);
	
	buf = get_buffer();
	fputs(buf,f);
	g_free(buf);

	if(errno)
		throw EIOError("Save file failed.",name,errno);
	
	fclose(f);
}

void GtkEditorView::get_pos(int &line,int &col) const
{
	GtkTextIter it;
	gtk_text_buffer_get_iter_at_mark(buffer,&it,"insert");

	line = gtk_text_iter_get_line_number(&it) + 1;
	col = gtk_text_iter_get_line_char(&it) + 1;
}

unsigned long GtkEditorView::get_pos(void) const
{
	GtkTextIter it;
	gtk_text_buffer_get_iter_at_mark(buffer,&it,"insert");
	
	return gtk_text_iter_get_char_index(&it);
}

void GtkEditorView::set_pos(int line,int col) const
{
	GtkTextIter it;
	gtk_text_iter_set_line_number(&it,line-1);
	gtk_text_iter_set_line_char(&it,col-1);

	gtk_text_buffer_move_mark(buffer,"insert",&it);
}

void GtkEditorView::set_pos(int pos) const
{
	GtkTextIter it;
	gtk_text_iter_set_char_index(&it,pos);

	gtk_text_buffer_move_mark(buffer,"insert",&it);
}

unsigned long GtkEditorView::get_char_count(void) const
{
	return gtk_text_buffer_get_char_count(buffer);
}

int GtkEditorView::get_line_count(void) const
{
	return gtk_text_buffer_get_line_count(buffer);
}

void GtkEditorView::clear(void) const
{
	int cnt = get_char_count()-1;
		
	if(cnt)
		gtk_text_buffer_delete_chars(buffer,0,cnt);
}

//ScintillaView
//internal
long ScintillaView::send_editor(unsigned int msg,unsigned long wparam,long lparam) const
{
    	return fn_editor(ptr_editor,msg,wparam,lparam);
}

long ScintillaView::send_editor_string(unsigned int msg,unsigned long wparam,const char *s) const
{
	return send_editor(msg,wparam,reinterpret_cast<long>(s));
}

void ScintillaView::get_range(int start,int end,char *text) const
{ 
	TextRange tr;
	tr.chrg.cpMin = start;
	tr.chrg.cpMax = end;
	tr.lpstrText = text;
	Platform::SendScintilla(w_editor.GetID(),SCI_GETTEXTRANGE,0,reinterpret_cast<long>(&tr));
}

//common
ScintillaView::ScintillaView()
{
	w_editor = scintilla_new();
	scintilla_set_id(SCINTILLA(w_editor.GetID()),300);
	fn_editor = reinterpret_cast<SciFnDirect>(Platform::SendScintilla(w_editor.GetID(),SCI_GETDIRECTFUNCTION,0,0));
	ptr_editor = Platform::SendScintilla(w_editor.GetID(),SCI_GETDIRECTPOINTER,0,0);
	gtk_widget_show_all(w_editor.GetID());
}

ScintillaView::~ScintillaView()
{
	gtk_widget_destroy(w_editor.GetID());
}

void ScintillaView::setup(const string &file)
{
	string lang = get_language(file);
	int lex;

	if(lang == "python"){
		lex = SCLEX_PYTHON;
	}else if(lang == "cpp"){
		lex = SCLEX_CPP;
	}else if(lang == "hypertext"){
		lex = SCLEX_HTML;
	}else if(lang == "xml"){
		lex = SCLEX_XML;
	}else if(lang == "perl"){
		lex = SCLEX_PERL;
	}else if(lang == "sql"){
		lex = SCLEX_SQL;
	}else if(lang == "vb"){
		lex = SCLEX_VB;
	}else if(lang == "props"){
		lex = SCLEX_PROPERTIES;
	}else if(lang == "errorlist"){
		lex = SCLEX_ERRORLIST;
	}else if(lang == "makefile"){
		lex = SCLEX_MAKEFILE;
	}else if(lang == "batch"){
		lex = SCLEX_BATCH;
	}else if(lang == "latex"){
		lex = SCLEX_LATEX;
	}else if(lang == "lua"){
		lex = SCLEX_LUA;
	}else if(lang == "diff"){
		lex = SCLEX_DIFF;
	}else if(lang == "container"){
		lex = SCLEX_CONTAINER;
	}else{
		lex = SCLEX_NULL;
	};

	if((lex == SCLEX_HTML)||(lex == SCLEX_XML))
		send_editor(SCI_SETSTYLEBITS,7);
	else
		send_editor(SCI_SETSTYLEBITS,5);

	send_editor(SCI_SETLEXER,lex);

	send_editor(SCI_STYLERESETDEFAULT,0,0);
	send_editor(SCI_STYLECLEARALL,0,0);

	char font[] = "courier";
	send_editor(SCI_STYLESETITALIC,STYLE_DEFAULT,0);
	send_editor(SCI_STYLESETBOLD,STYLE_DEFAULT,0);
	send_editor(SCI_STYLESETFONT,STYLE_DEFAULT,(long)font);
	send_editor(SCI_STYLESETFORE,STYLE_DEFAULT,0x000000);
	send_editor(SCI_STYLESETBACK,STYLE_DEFAULT,0xFFFFFF);
	send_editor(SCI_STYLESETSIZE,STYLE_DEFAULT,12);
	send_editor(SCI_STYLESETUNDERLINE,STYLE_DEFAULT,0);

	if(lang == "cpp"){
		send_editor_string(SCI_SETKEYWORDS,0,"asm auto bool break case catch char class const const_cast continue default delete do double dynamic_cast else enum explicit export extern false float for friend goto if inline int long mutable namespace new operator private protected public register reinterpret_cast return short signed sizeof static static_cast struct switch template this throw true try typedef typeid typename union unsigned using virtual void volatile wchar_t while");
		
		//Whitespace
		send_editor(SCI_STYLESETFONT,0,(long)font);
		send_editor(SCI_STYLESETFORE,0,0x808080);
		
		//Comment
		send_editor(SCI_STYLESETFONT,1,(long)font);
		send_editor(SCI_STYLESETFORE,1,0x0000FF);
		send_editor(SCI_STYLESETITALIC,1,1);
		
		//Line Comment
		send_editor(SCI_STYLESETFONT,2,(long)font);
		send_editor(SCI_STYLESETFORE,2,0x0000AA);
		send_editor(SCI_STYLESETITALIC,2,1);

		//Doc Commenet
		send_editor(SCI_STYLESETFONT,3,(long)font);
		send_editor(SCI_STYLESETFORE,3,0x0000AA);
		send_editor(SCI_STYLESETITALIC,3,1);
		
		//Number
		send_editor(SCI_STYLESETFONT,4,(long)font);
		
		//Keyword
		send_editor(SCI_STYLESETFONT,5,(long)font);
		send_editor(SCI_STYLESETFORE,5,0xFF0000);
		send_editor(SCI_STYLESETBOLD,5,1);
	
		//Double quoted string
		send_editor(SCI_STYLESETFONT,6,(long)font);
		send_editor(SCI_STYLESETFORE,6,0x00CC00);
		
		//Single quoted string
		send_editor(SCI_STYLESETFONT,7,(long)font);
		send_editor(SCI_STYLESETFORE,7,0x00FF00);
		
		//UUID
		//Preprocessor
		send_editor(SCI_STYLESETFONT,9,(long)font);
		send_editor(SCI_STYLESETFORE,9,0x7F7F00);
		
		//Operators
		send_editor(SCI_STYLESETFONT,10,(long)font);
		
		//Identifiers
		send_editor(SCI_STYLESETFONT,11,(long)font);
		
		//EOL string not closed
		//Verbatim strings C#
	};

	send_editor(SCI_COLOURISE,0,-1);
}	

long ScintillaView::get_sci_color(const string &triplet) const
{
	unsigned char r,g,b;

	triplet2RGB(triplet,r,g,b);
	return r + 256*g + 65536*b;
}

GtkWidget *ScintillaView::get_widget(void)
{
	return w_editor.GetID();
}

char *ScintillaView::get_buffer(void) const
{
}

bool ScintillaView::modified(void) const
{
	send_editor(SCI_GETMODIFY);
}

void ScintillaView::set_settings(const ViewSettings &set) const
{
	send_editor(SCI_SETTABWIDTH,set.tab_stop);
	send_editor(SCI_SETUSETABS,!set.spaces);
	send_editor(SCI_SETVIEWWS,set.viewws);
	send_editor(SCI_SETVIEWEOL,set.vieweol);
	send_editor(SCI_SETINDENTATIONGUIDES,set.indentation);

	send_editor(SCI_SETSELFORE,true,get_sci_color(set.selcolor));
	send_editor(SCI_SETSELBACK,true,get_sci_color(set.selback));
}

void ScintillaView::load(const string &name)
{
	setup(name);	

	send_editor(SCI_CANCEL);
	send_editor(SCI_SETUNDOCOLLECTION,0);

	FILE *fp = fopen(name.c_str(), "rb");
	if(fp){
		char data[BUF_MAX_SIZE];
		int len = fread(data,1,sizeof(data),fp);
		while(len>0){
			send_editor_string(SCI_ADDTEXT,len,data);
			len = fread(data,1,sizeof(data),fp);
		};
		fclose(fp);
	};
	send_editor(SCI_SETUNDOCOLLECTION,1);
	send_editor(SCI_EMPTYUNDOBUFFER);
	send_editor(SCI_SETSAVEPOINT);
	send_editor(SCI_GOTOPOS,0);
	
	w_editor.InvalidateAll();
}

void ScintillaView::save(const string &name) const
{
    	FILE *fp = fopen(name.c_str(),"wb");
	if(fp){
		char data[BUF_MAX_SIZE + 1];
		int len = get_char_count();
		for(int i = 0;i < len;i += BUF_MAX_SIZE){
	    		int grab = len - i;
			if (grab > BUF_MAX_SIZE)
				grab = BUF_MAX_SIZE;
			get_range(i,i + grab,data);
			fwrite(data,grab,1,fp);
		};	
	};
	fclose(fp);
	send_editor(SCI_SETSAVEPOINT);
}

void ScintillaView::get_pos(int &line,int &col) const
{
}

unsigned long ScintillaView::get_pos(void) const
{
	return send_editor(SCI_GETCURRENTPOS);
}

void ScintillaView::set_pos(int line,int col) const
{
	//col parameter is ignored
	send_editor(SCI_GOTOLINE,line-1);
}

void ScintillaView::set_pos(int pos) const
{
	send_editor(SCI_GOTOPOS,pos-1);
}

unsigned long ScintillaView::get_char_count(void) const
{
	return send_editor(SCI_GETLENGTH);
}

int ScintillaView::get_line_count(void) const
{
	return send_editor(SCI_GETLINECOUNT);
}

void ScintillaView::clear(void) const
{
}

void ScintillaView::undo(void) const
{
	send_editor(SCI_UNDO);
}

void ScintillaView::redo(void) const
{
	send_editor(SCI_REDO);
}

void ScintillaView::cut(void) const
{
	send_editor(SCI_CUT);
}

void ScintillaView::copy(void) const
{
	send_editor(SCI_COPY);
}

void ScintillaView::paste(void) const
{
	send_editor(SCI_PASTE);
}

void ScintillaView::insert(const string &text) const
{
	send_editor(SCI_INSERTTEXT,get_pos(),reinterpret_cast<long>(text.c_str()));
}
